<?php
// no direct access
defined('_JEXEC') or die('Restricted access');

/* * ***********************************************************************
 * File App for Watchful
 * Copyright (c) 2006-2013 Anything Digital & Inetis
 * ************************************************************************
 * Watchful.li is a web site monitoring and security service.
 * 
 * Although Watchful.li is a paid service, this program is free software;
 * you can redistribute it and/or modify it under the terms of the GNU 
 * General Public License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option) any later version.
 *
 * This header must not be removed. Additional contributions/changes
 * may be added to this header as long as no information is deleted.
 * ************************************************************************
 * Get the latest version of this program at: https://watchful.li
 * *********************************************************************** */

require_once(JPATH_ADMINISTRATOR.'/components/com_watchfulli/classes/apps.php');
// We extend a base class to pass additional parameters in the language strings
// It will probably be included in the client but we also put it here so the app will work
// even for older version of the Watchful client.
class FileAppAlert extends AppAlert
{
    public $parameter1;
    public $parameter2;
    public $parameter3;

    public function FileAppAlert($level, $message, $parameter1 = null, $parameter2 = null, $parameter3 = null)
    {
        if ($level != null && $message != null)
        {
            $this->level = $level;
            $this->message = $message;
            $this->parameter1 = $parameter1;
            $this->parameter2 = $parameter2;
            $this->parameter3 = $parameter3;
        }
    }

}
class plgWatchfulliAppsFileApp extends watchfulliApps
{

    public function createAppAlert($level, $message, $parameter1 = null, $parameter2 = null, $parameter3 = null)
    {
        $alert = new FileAppAlert($level, $message, $parameter1, $parameter2, $parameter3);
        $this->addAlert($alert);
    }

    public function appMainProgram($oldValuesSerialized = null)
    {
        $this->setName("fileApp");
        $this->setDescription("File App for Watchful - This App checks for modification date and checksum of a file.");
        $debug = $this->params->get('debug', 0);

        // Get all parameters as array
        $files = array();
        for ($i = 1; $i <= 10; $i++)
        {
            $file = $this->params->get("file$i", '');
            if ($file)
            {
                $files[] = $file;
                $filesProperties[$file] = $this->fileCheck($file);
            }
        }
        unset($file);

        // Check that at least one parameter is set
        if (empty($files))
        {
            if ($debug)
                $this->createAppAlert(1, "[DEBUG][FILEAPP] No parameter set");
            return $this;
        }

        // List of files to check
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][FILEAPP] File(s) to check: " . join(' - ', $files));

        // Build new value
        if ($filesProperties)
        {
            $newValue = json_encode($filesProperties);
        }
        else
        {
            $newValue = "[]";
        }
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][FILEAPP] New value is: " . $newValue);

        // Get the old value
        $oldValue = $this->readExAppValue($oldValuesSerialized);
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][FILEAPP] Old value is: " . $oldValue);

        if ($oldValue == null)
        {
            $this->createAppValue('fileApp', $newValue);
            if ($debug)
                $this->createAppAlert(1, "[DEBUG][FILEAPP] File App executed for the first time.");
        }
        else
        {
            // we should check each value against the previous one
            $changed = false;
            $oldFilesProperties = json_decode($oldValue);
            foreach ($filesProperties as $file => $properties)
            {
                if (!isset($oldFilesProperties->$file))
                {
                    $changed = true;
                    $this->createAppAlert(1, "FILEAPP_FILE_ADDED", $file); // A new file has been added to the check list
                }
                else if (json_encode($oldFilesProperties->$file) != json_encode($properties))
                {
                    $changed = true;
                    $this->createAppAlert(1, "FILEAPP_FILE_CHANGED", $file);
                }
            }
            if ($changed)
                $this->updateAppValue('fileApp', $newValue);
        }

        return $this;
    }

    public function fileCheck($file)
    {
        if (file_exists(JPATH_SITE . $file))
        {
            $fp = fopen(JPATH_SITE . $file, 'r');
            $fstat = fstat($fp);
            fclose($fp);
            $checksum = md5_file(JPATH_SITE . $file);
        }
        elseif ($file != JPATH_ROOT . '/.htaccess')
        { //If not, we say that the file can't be found
            $checksum = $fstat['size'] = $fstat['mtime'] = 'NOT_FOUND';
        }
        return array('rootpath' => $file, 'size' => $fstat['size'], 'modificationtime' => $fstat['mtime'], 'checksum' => $checksum);
    }

}
